import { Span } from 'opentracing';
import { Observable } from 'rxjs';
import { LanguageClient } from './lang-handler';
import { FileSystemEntries } from './match-files';
/**
 * Provides a synchronous file system API.
 */
export interface SynchronousFileSystem {
    /**
     * Returns true if the given file is known to exist in the workspace (content loaded or not)
     *
     * @param uri URI to a file
     */
    fileExists(uri: string): boolean;
    /**
     * Returns the file content for the given URI, if that file is synchronously available.
     */
    readFileIfAvailable(uri: string): string | undefined;
    /**
     * Return the files and directories contained in the given directory
     */
    getFileSystemEntries(directory: string): FileSystemEntries;
}
/**
 * In-memory file cache node which represents either a folder or a file
 */
export interface FileSystemNode {
    file: boolean;
    children: Map<string, FileSystemNode>;
}
/**
 * In-memory file system
 */
export declare class InMemoryFileSystem implements SynchronousFileSystem {
    private readonly files;
    /**
     * File tree root
     */
    rootNode: FileSystemNode;
    /**
     * Contains a Map of all URIs that exist in the workspace, optionally with a content.
     * File contents for URIs in it do not neccessarily have to be fetched already.
     */
    constructor(files?: Map<string, string | undefined>);
    /**
     * Return all known uri's in the workspace. (content loaded or not)
     */
    uris(): IterableIterator<string>;
    /**
     * Adds a file to the local cache
     *
     * @param uri The URI of the file
     * @param content The optional content
     */
    add(uri: string, content?: string): void;
    /**
     * Return the files and directories contained in the given directory
     */
    getFileSystemEntries(directory: string): FileSystemEntries;
    /**
     * Returns true if the given file is known to exist in the workspace (content loaded or not)
     *
     * @param uri URI to a file
     */
    fileExists(uri: string): boolean;
    /**
     * Returns the file content for the given URI.
     */
    readFileIfAvailable(uri: string): string | undefined;
}
export declare class LocalFileSystem implements SynchronousFileSystem {
    /**
     * Returns true if the given file is known to exist in the workspace (content loaded or not)
     * @param uri URI to a file
     */
    fileExists(uri: string): boolean;
    /**
     * Returns the file content for the given URI.
     */
    readFileIfAvailable(uri: string): string | undefined;
    /**
     * Return the files and directories contained in the given directory
     */
    getFileSystemEntries(directory: string): FileSystemEntries;
}
/**
 * Provides a minimal asynchronous file system API.
 */
export interface AsynchronousFileSystem {
    /**
     * Returns all files in the workspace under base
     *
     * @param base A URI under which to search, resolved relative to the rootUri
     * @return An Observable that emits URIs
     */
    getWorkspaceFiles(base?: string | undefined, childOf?: Span | undefined): Observable<string>;
    /**
     * Returns the content of a text document
     *
     * @param uri The URI of the text document, resolved relative to the rootUri
     * @return An Observable that emits the text document content
     */
    getTextDocumentContent(uri: string, childOf?: Span | undefined): Observable<string>;
}
export declare class RemoteFileSystem implements AsynchronousFileSystem {
    private client;
    constructor(client: LanguageClient);
    /**
     * The files request is sent from the server to the client to request a list of all files in the workspace or inside the directory of the base parameter, if given.
     * A language server can use the result to index files by filtering and doing a content request for each text document of interest.
     */
    getWorkspaceFiles(base?: string, childOf?: Span): Observable<string>;
    /**
     * The content request is sent from the server to the client to request the current content of
     * any text document. This allows language servers to operate without accessing the file system
     * directly.
     */
    getTextDocumentContent(uri: string, childOf?: Span): Observable<string>;
}
