"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("mz/fs");
const opentracing_1 = require("opentracing");
const path = require("path");
const util_1 = require("./util");
/**
 * In-memory file system
 */
class InMemoryFileSystem {
    /**
     * Contains a Map of all URIs that exist in the workspace, optionally with a content.
     * File contents for URIs in it do not neccessarily have to be fetched already.
     */
    constructor(files = new Map()) {
        this.files = files;
        /**
         * File tree root
         */
        this.rootNode = { file: false, children: new Map() };
    }
    /**
     * Return all known uri's in the workspace. (content loaded or not)
     */
    uris() {
        return this.files.keys();
    }
    /**
     * Adds a file to the local cache
     *
     * @param uri The URI of the file
     * @param content The optional content
     */
    add(uri, content) {
        // Make sure not to override existing content with undefined
        if (content !== undefined || !this.files.has(uri)) {
            this.files.set(uri, content);
        }
        // Add to directory tree
        // TODO: convert this to use URIs.
        const filePath = util_1.uri2path(uri);
        const components = filePath.split(/[\/\\]/).filter(c => c);
        let node = this.rootNode;
        for (const [i, component] of components.entries()) {
            const n = node.children.get(component);
            if (!n) {
                if (i < components.length - 1) {
                    const n = { file: false, children: new Map() };
                    node.children.set(component, n);
                    node = n;
                }
                else {
                    node.children.set(component, { file: true, children: new Map() });
                }
            }
            else {
                node = n;
            }
        }
    }
    /**
     * Return the files and directories contained in the given directory
     */
    getFileSystemEntries(directory) {
        const ret = { files: [], directories: [] };
        let node = this.rootNode;
        const components = directory.split(/[\\\/]/).filter(c => c);
        if (components.length !== 1 || components[0]) {
            for (const component of components) {
                const n = node.children.get(component);
                if (!n) {
                    return ret;
                }
                node = n;
            }
        }
        for (const [name, value] of node.children.entries()) {
            if (value.file) {
                ret.files.push(name);
            }
            else {
                ret.directories.push(name);
            }
        }
        return ret;
    }
    /**
     * Returns true if the given file is known to exist in the workspace (content loaded or not)
     *
     * @param uri URI to a file
     */
    fileExists(uri) {
        return this.files.has(uri);
    }
    /**
     * Returns the file content for the given URI.
     */
    readFileIfAvailable(uri) {
        return this.files.get(uri);
    }
}
exports.InMemoryFileSystem = InMemoryFileSystem;
class LocalFileSystem {
    /**
     * Returns true if the given file is known to exist in the workspace (content loaded or not)
     * @param uri URI to a file
     */
    fileExists(uri) {
        return fs.existsSync(util_1.uri2path(uri));
    }
    /**
     * Returns the file content for the given URI.
     */
    readFileIfAvailable(uri) {
        try {
            return fs.readFileSync(util_1.uri2path(uri), 'utf8');
        }
        catch (e) {
            return undefined;
        }
    }
    /**
     * Return the files and directories contained in the given directory
     */
    getFileSystemEntries(directory) {
        const files = [];
        const directories = [];
        for (const name of fs.readdirSync(directory)) {
            const filePath = path.join(directory, name);
            try {
                const stat = fs.statSync(filePath);
                if (stat.isFile()) {
                    files.push(name);
                }
                else if (stat.isDirectory()) {
                    directories.push(name);
                }
            }
            catch (e) {
                // no-op
            }
        }
        return { files, directories };
    }
}
exports.LocalFileSystem = LocalFileSystem;
class RemoteFileSystem {
    constructor(client) {
        this.client = client;
    }
    /**
     * The files request is sent from the server to the client to request a list of all files in the workspace or inside the directory of the base parameter, if given.
     * A language server can use the result to index files by filtering and doing a content request for each text document of interest.
     */
    getWorkspaceFiles(base, childOf = new opentracing_1.Span()) {
        return this.client
            .workspaceXfiles({ base }, childOf)
            .mergeMap(textDocuments => textDocuments)
            .map(textDocument => util_1.normalizeUri(textDocument.uri));
    }
    /**
     * The content request is sent from the server to the client to request the current content of
     * any text document. This allows language servers to operate without accessing the file system
     * directly.
     */
    getTextDocumentContent(uri, childOf = new opentracing_1.Span()) {
        return this.client
            .textDocumentXcontent({ textDocument: { uri } }, childOf)
            .map(textDocument => textDocument.text);
    }
}
exports.RemoteFileSystem = RemoteFileSystem;
//# sourceMappingURL=fs.js.map