/// <reference types="node" />
import { EventEmitter } from 'events';
import * as ts from 'typescript';
import { SynchronousFileSystem } from './fs';
import { Logger } from './logging';
import { FileSystemEntries } from './match-files';
/**
 * TypeScript library files fetched from the local file system (bundled TS)
 */
export declare const typeScriptLibraries: Map<string, string>;
/**
 * Overlay a in-memory list of files on top of an existing file system. Useful for storing file state for files that are being editted. Can be served as a ParseConfigHost (thus allowing listing files that belong to project based on tsconfig.json options)
 */
export declare class OverlayFileSystem extends EventEmitter implements ts.ParseConfigHost, ts.ModuleResolutionHost {
    readonly fileSystem: SynchronousFileSystem;
    private logger;
    /**
     * Map (URI -> string content) of temporary files made while user modifies local file(s)
     */
    overlay: Map<string, string>;
    /**
     * Should we take into account register when performing a file name match or not. On Windows when using local file system, file names are case-insensitive
     */
    useCaseSensitiveFileNames: boolean;
    /**
     * Root path
     */
    path: string;
    constructor(fileSystem: SynchronousFileSystem, path: string, logger?: Logger);
    /**
     * Tells if a file denoted by the given name exists in the workspace (does not have to be loaded)
     *
     * @param path File path or URI (both absolute or relative file paths are accepted)
     */
    fileExists(path: string): boolean;
    /**
     * @param path file path (both absolute or relative file paths are accepted)
     * @return file's content in the following order (overlay then cache).
     * If there is no such file, returns empty string to match expected signature
     */
    readFile(path: string): string;
    /**
     * @param path file path (both absolute or relative file paths are accepted)
     * @return file's content in the following order (overlay then cache).
     * If there is no such file, returns undefined
     */
    readFileIfExists(path: string): string | undefined;
    /**
     * Returns the file content for the given URI.
     * Will throw an Error if no available in-memory.
     * Use FileSystemUpdater.ensure() to ensure that the file is available.
     */
    getContent(uri: string): string;
    /**
     * Invalidates temporary content denoted by the given URI
     * @param uri file's URI
     */
    didClose(uri: string): void;
    /**
     * Adds temporary content denoted by the given URI
     * @param uri file's URI
     */
    didSave(uri: string): void;
    /**
     * Updates temporary content denoted by the given URI
     * @param uri file's URI
     */
    didChange(uri: string, text: string): void;
    /**
     * Called by TS service to scan virtual directory when TS service looks for source files that belong to a project
     */
    readDirectory(rootDir: string, extensions: ReadonlyArray<string> | undefined, excludes: ReadonlyArray<string> | undefined, includes: ReadonlyArray<string>): string[];
    /**
     * Return the files and directories contained in the given directory
     */
    getFileSystemEntries(path: string): FileSystemEntries;
    trace(message: string): void;
}
/**
 * @param path file path
 * @return true if given file belongs to bundled TypeScript libraries
 */
export declare function isTypeScriptLibrary(path: string): boolean;
