"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const fs = require("fs");
const path = require("path");
const ts = require("typescript");
const fs_1 = require("./fs");
const logging_1 = require("./logging");
const match_files_1 = require("./match-files");
const util_1 = require("./util");
/**
 * TypeScript library files fetched from the local file system (bundled TS)
 */
exports.typeScriptLibraries = new Map();
/**
 * Overlay a in-memory list of files on top of an existing file system. Useful for storing file state for files that are being editted. Can be served as a ParseConfigHost (thus allowing listing files that belong to project based on tsconfig.json options)
 */
class OverlayFileSystem extends events_1.EventEmitter {
    constructor(fileSystem, path, logger = new logging_1.NoopLogger()) {
        super();
        this.fileSystem = fileSystem;
        this.logger = logger;
        this.path = path;
        this.overlay = new Map();
    }
    /**
     * Tells if a file denoted by the given name exists in the workspace (does not have to be loaded)
     *
     * @param path File path or URI (both absolute or relative file paths are accepted)
     */
    fileExists(path) {
        const uri = util_1.path2uri(path);
        return this.overlay.has(uri) || this.fileSystem.fileExists(uri) || exports.typeScriptLibraries.has(path);
    }
    /**
     * @param path file path (both absolute or relative file paths are accepted)
     * @return file's content in the following order (overlay then cache).
     * If there is no such file, returns empty string to match expected signature
     */
    readFile(path) {
        const content = this.readFileIfExists(path);
        if (content === undefined) {
            this.logger.warn(`readFile ${path} requested by TypeScript but content not available`);
            return '';
        }
        return content;
    }
    /**
     * @param path file path (both absolute or relative file paths are accepted)
     * @return file's content in the following order (overlay then cache).
     * If there is no such file, returns undefined
     */
    readFileIfExists(path) {
        const uri = util_1.path2uri(path);
        let content = this.overlay.get(uri);
        if (content !== undefined) {
            return content;
        }
        // TODO This assumes that the URI was a file:// URL.
        //      In reality it could be anything, and the first URI matching the path should be used.
        //      With the current Map, the search would be O(n), it would require a tree to get O(log(n))
        content = this.fileSystem.readFileIfAvailable(uri);
        if (content !== undefined) {
            return content;
        }
        return exports.typeScriptLibraries.get(path);
    }
    /**
     * Returns the file content for the given URI.
     * Will throw an Error if no available in-memory.
     * Use FileSystemUpdater.ensure() to ensure that the file is available.
     */
    getContent(uri) {
        const content = this.readFileIfExists(util_1.uri2path(uri));
        if (content === undefined) {
            throw new Error(`Content of ${uri} is not available in memory`);
        }
        return content;
    }
    /**
     * Invalidates temporary content denoted by the given URI
     * @param uri file's URI
     */
    didClose(uri) {
        this.overlay.delete(uri);
    }
    /**
     * Adds temporary content denoted by the given URI
     * @param uri file's URI
     */
    didSave(uri) {
        const content = this.overlay.get(uri);
        if (content !== undefined && this.fileSystem instanceof fs_1.InMemoryFileSystem) {
            this.fileSystem.add(uri, content);
        }
    }
    /**
     * Updates temporary content denoted by the given URI
     * @param uri file's URI
     */
    didChange(uri, text) {
        this.overlay.set(uri, text);
    }
    /**
     * Called by TS service to scan virtual directory when TS service looks for source files that belong to a project
     */
    readDirectory(rootDir, extensions, excludes, includes) {
        return match_files_1.matchFiles(rootDir, extensions, excludes, includes, true, this.path, p => this.fileSystem.getFileSystemEntries(p));
    }
    /**
     * Return the files and directories contained in the given directory
     */
    getFileSystemEntries(path) {
        return this.fileSystem.getFileSystemEntries(path);
    }
    trace(message) {
        this.logger.log(message);
    }
}
exports.OverlayFileSystem = OverlayFileSystem;
/**
 * Fetching TypeScript library files from local file system
 */
const libPath = path.dirname(ts.getDefaultLibFilePath({ target: ts.ScriptTarget.ES2015 }));
for (const file of fs.readdirSync(libPath)) {
    const fullPath = path.join(libPath, file);
    if (fs.statSync(fullPath).isFile()) {
        exports.typeScriptLibraries.set(util_1.toUnixPath(fullPath), fs.readFileSync(fullPath).toString());
    }
}
/**
 * @param path file path
 * @return true if given file belongs to bundled TypeScript libraries
 */
function isTypeScriptLibrary(path) {
    return exports.typeScriptLibraries.has(util_1.toUnixPath(path));
}
exports.isTypeScriptLibrary = isTypeScriptLibrary;
//# sourceMappingURL=memfs.js.map