import { Span } from 'opentracing';
import { Observable } from 'rxjs';
import { AsynchronousFileSystem, InMemoryFileSystem } from './fs';
export interface FileSystemUpdater {
    knownUrisWithoutAvailableContent(): IterableIterator<string>;
    ensure(uri: string, childOf?: Span): Observable<never>;
    ensureStructure(childOf?: Span): Observable<never>;
}
export declare class NoopFileSystemUpdater implements FileSystemUpdater {
    knownUrisWithoutAvailableContent(): IterableIterator<string>;
    ensure(uri: string, childOf?: Span | undefined): Observable<never>;
    ensureStructure(childOf?: Span | undefined): Observable<never>;
}
/**
 * Synchronizes a remote file system to an in-memory file system
 *
 * TODO: Implement Disposable with Disposer
 */
export declare class RemoteFileSystemUpdater implements FileSystemUpdater {
    readonly remoteFileSystem: AsynchronousFileSystem;
    readonly fileSystem: InMemoryFileSystem;
    /**
     * Observable for a pending or completed structure fetch
     */
    private structureFetch?;
    /**
     * Map from URI to Observable of pending or completed content fetch
     */
    private fetches;
    /**
     * Limits concurrent fetches to not fetch thousands of files in parallel
     */
    private concurrencyLimit;
    constructor(remoteFileSystem: AsynchronousFileSystem, fileSystem: InMemoryFileSystem);
    /**
     * Fetches the file content for the given URI and adds the content to the in-memory file system
     *
     * @param uri URI of the file to fetch
     * @param childOf A parent span for tracing
     * @return Observable that completes when the fetch is finished
     */
    fetch(uri: string, childOf?: Span): Observable<never>;
    /**
     * Returns a promise that is resolved when the given URI has been fetched (at least once) to the in-memory file system.
     * This function cannot be cancelled because multiple callers get the result of the same operation.
     *
     * @param uri URI of the file to ensure
     * @param childOf An OpenTracing span for tracing
     * @return Observable that completes when the file was fetched
     */
    ensure(uri: string, childOf?: Span): Observable<never>;
    /**
     * Fetches the file/directory structure for the given directory from the remote file system and saves it in the in-memory file system
     *
     * @param childOf A parent span for tracing
     */
    fetchStructure(childOf?: Span): Observable<never>;
    /**
     * Returns a promise that is resolved as soon as the file/directory structure for the given directory has been synced
     * from the remote file system to the in-memory file system (at least once)
     *
     * @param span An OpenTracing span for tracing
     */
    ensureStructure(childOf?: Span): Observable<never>;
    /**
     * Invalidates the content fetch cache of a file.
     * The next call to `ensure` will do a refetch.
     *
     * @param uri URI of the file that changed
     */
    invalidate(uri: string): void;
    /**
     * Invalidates the structure fetch cache.
     * The next call to `ensureStructure` will do a refetch.
     */
    invalidateStructure(): void;
    /**
     * Returns an IterableIterator for all URIs known to exist in the workspace whose content is not synchronously available.
     */
    knownUrisWithoutAvailableContent(): IterableIterator<string>;
}
