"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const opentracing_1 = require("opentracing");
const rxjs_1 = require("rxjs");
const Semaphore_1 = require("semaphore-async-await/dist/Semaphore");
const tracing_1 = require("./tracing");
class NoopFileSystemUpdater {
    *knownUrisWithoutAvailableContent() {
        // no-op
    }
    ensure(uri, childOf) {
        return rxjs_1.Observable.empty();
    }
    ensureStructure(childOf) {
        return rxjs_1.Observable.empty();
    }
}
exports.NoopFileSystemUpdater = NoopFileSystemUpdater;
/**
 * Synchronizes a remote file system to an in-memory file system
 *
 * TODO: Implement Disposable with Disposer
 */
class RemoteFileSystemUpdater {
    constructor(remoteFileSystem, fileSystem) {
        this.remoteFileSystem = remoteFileSystem;
        this.fileSystem = fileSystem;
        /**
         * Map from URI to Observable of pending or completed content fetch
         */
        this.fetches = new Map();
        /**
         * Limits concurrent fetches to not fetch thousands of files in parallel
         */
        this.concurrencyLimit = new Semaphore_1.default(100);
    }
    /**
     * Fetches the file content for the given URI and adds the content to the in-memory file system
     *
     * @param uri URI of the file to fetch
     * @param childOf A parent span for tracing
     * @return Observable that completes when the fetch is finished
     */
    fetch(uri, childOf = new opentracing_1.Span()) {
        // Limit concurrent fetches
        const observable = rxjs_1.Observable.fromPromise(this.concurrencyLimit.wait())
            .mergeMap(() => this.remoteFileSystem.getTextDocumentContent(uri))
            .do(content => {
            this.concurrencyLimit.signal();
            this.fileSystem.add(uri, content);
        }, err => {
            this.fetches.delete(uri);
        })
            .ignoreElements()
            .publishReplay()
            .refCount();
        this.fetches.set(uri, observable);
        return observable;
    }
    /**
     * Returns a promise that is resolved when the given URI has been fetched (at least once) to the in-memory file system.
     * This function cannot be cancelled because multiple callers get the result of the same operation.
     *
     * @param uri URI of the file to ensure
     * @param childOf An OpenTracing span for tracing
     * @return Observable that completes when the file was fetched
     */
    ensure(uri, childOf = new opentracing_1.Span()) {
        return tracing_1.traceObservable('Ensure content', childOf, span => {
            span.addTags({ uri });
            return this.fetches.get(uri) || this.fetch(uri, span);
        });
    }
    /**
     * Fetches the file/directory structure for the given directory from the remote file system and saves it in the in-memory file system
     *
     * @param childOf A parent span for tracing
     */
    fetchStructure(childOf = new opentracing_1.Span()) {
        const observable = tracing_1.traceObservable('Fetch workspace structure', childOf, span => this.remoteFileSystem
            .getWorkspaceFiles(undefined, span)
            .do(uri => {
            this.fileSystem.add(uri);
        }, err => {
            this.structureFetch = undefined;
        })
            .ignoreElements()
            .publishReplay()
            .refCount());
        this.structureFetch = observable;
        return observable;
    }
    /**
     * Returns a promise that is resolved as soon as the file/directory structure for the given directory has been synced
     * from the remote file system to the in-memory file system (at least once)
     *
     * @param span An OpenTracing span for tracing
     */
    ensureStructure(childOf = new opentracing_1.Span()) {
        return tracing_1.traceObservable('Ensure structure', childOf, span => this.structureFetch || this.fetchStructure(span));
    }
    /**
     * Invalidates the content fetch cache of a file.
     * The next call to `ensure` will do a refetch.
     *
     * @param uri URI of the file that changed
     */
    invalidate(uri) {
        this.fetches.delete(uri);
    }
    /**
     * Invalidates the structure fetch cache.
     * The next call to `ensureStructure` will do a refetch.
     */
    invalidateStructure() {
        this.structureFetch = undefined;
    }
    /**
     * Returns an IterableIterator for all URIs known to exist in the workspace whose content is not synchronously available.
     */
    *knownUrisWithoutAvailableContent() {
        for (const file of this.fileSystem.uris()) {
            if (!this.fileSystem.readFileIfAvailable(file)) {
                yield file;
            }
        }
    }
}
exports.RemoteFileSystemUpdater = RemoteFileSystemUpdater;
//# sourceMappingURL=updater.js.map